@@ -1,5 +1,7 @@
 # trigger patterns:
+MC_MONO_TRIGGER_PATTERN = 0
 DATA_MONO_TRIGGER_PATTERN = 1
 MC_STEREO_TRIGGER_PATTERN = 1
 PEDESTAL_TRIGGER_PATTERN = 8
+MC_SUMT_TRIGGER_PATTERN = 32
 DATA_STEREO_TRIGGER_PATTERN = 128
